#
# Script version 1.3
#
# Version history:
#  1.2  (start of version history tracking)
#  1.3  (NOA according to the prefix of the calling/called number, manipulations on both calling/called number. For calling number, if privacy exist, use that as the calling number to do the NOA and manipulation ) 
#
require 'base_routing'

# This script demonstrates how to peform a remapping of the Called NOA and NPI based on the Called number prefix (001 for international).
# It also routes calls in the most simple fashion. The routing behavior is the same as routing with 'SimpleRouting'.
#
# Routing is in the following order:
#
# * Matching: <tt>BaseRouting.route_match</tt>
#   * <tt>:call_field_name => :called</tt> - Match the called number of the call to a route.
#   * <tt>:call_field_name => :calling</tt> - Match the calling number of the call to a route.
#   * <tt>:call_field_name => :nap</tt> - Match the nap of the call to a route.
#   * <tt>:method => :match_nap_availability</tt> - Match the call to a route using BaseRouting.match_nap_availability.
# * Remapping: <tt>BaseRouting.route_remap</tt>
#   * <tt>:call_field_name => :called, :route_field_name => :remapped_called</tt> - Remap the called
#     number for the outgoing call with the choosen route field named 'remapped_called'.
#   * <tt>:call_field_name => :calling, :route_field_name => :remapped_calling</tt> - Remap the calling
#     number for the outgoing call with the choosen route field named 'remapped_calling'.
#   * <tt>:call_field_name => :nap, :route_field_name => :remapped_nap</tt> - Remap the nap for the outgoing call.
#     This means setting the destination nap for the route.
#   * <tt>:method => :remap_called_noa_npi</tt> - NOA and NPI remapping based on the Called number prefix.
#
#


class NoaNpiRemap < BaseRouting
  route_match :call_field_name => :called
  route_match :call_field_name => :calling
  route_match :call_field_name => :nap
  #  route_match :method => :match_nap_availability
  route_remap :method => :prefix_private_noa
  route_remap :call_field_name => :called, :route_field_name => :remapped_called
  route_remap :call_field_name => :calling, :route_field_name => :remapped_calling
  route_remap :call_field_name => :nap, :route_field_name => :remapped_nap

  # The route_remap method allow easy cutomization of outgoing call parameters remapping.
  #
  # This function takes the following arguments as parameters:
  # * <tt>route </tt> - Hash of route parameters. Chosen route during the 'matching' phase of the BaseRouting::route function.
  # * <tt>caf_call </tt> - Caf call object.
  # * <tt>nap_list </tt> - Hash of NAP statuses and custom fields (Create New NAP Column).
  # * <tt>call_params </tt> - Hash of call parameters. This hash contains all the call parameters. The remapped call parameter must be
  # set in this hash. This hash also contains all previously remapped call parameter by other 'route_remap' functions.
  # In this example, the 'remap_called_noa_npi' method is called before the called, calling and nap remapping.
  #
  # This function must return the remapped call parameters in the call_param hash.
  #
  # This method remap the Called number NOA and NPI based on a prefix rules regex.
  # * If the called number match '001', the NOA is remapped to 'international_number'.
  # * Otherwise, the NOA is remapped to 'national_number'
  # * The NPI is always remapped to 'isdn'.
  #
  # NOA values for the <tt>called_noa</tt> or <tt>calling_noa</tt> remapped call parameter are:
  # * <tt>unknown_number</tt>
  # * <tt>international_number</tt>
  # * <tt>national_number</tt>
  # * <tt>subscriber_number</tt>
  # * <tt>network_specific</tt>
  # * <tt>network_routing_national_format</tt>
  # * <tt>network_routing_international_format</tt>
  # * <tt>abbreviated_number</tt>
  # * <tt>subscriber_number_operator_requested</tt>
  # * <tt>national_number_operator_requested</tt>
  # * <tt>international_number_operator_requested</tt>
  # * <tt>no_number_present_operator_requested</tt>
  # * <tt>no_number_present_cut_through_call_to_carrier</tt>
  # * <tt>test_line_test_code</tt>
  # * <tt>non_unique_subscriber_number</tt>
  # * <tt>non_unique_national_number</tt>
  # * <tt>non_unique_international_number</tt>
  # * <tt>call_950_number</tt>
  #
  # Those values will be remapped to the protocol specific NOA value. To provide protocol specific value:
  # * <tt>remapped_fields[:called_noa] = 0x70</tt>
  # or
  # * <tt>remapped_fields[:called_noa] = 112</tt>
  #
  # NPI values for the <tt>called_npi</tt> or <tt>calling_npi</tt> remmapped call parameter are:
  # * <tt>unknown_number</tt>
  # * <tt>isdn</tt>
  # * <tt>telephony</tt>
  # * <tt>private</tt>
  # * <tt>data</tt>
  # * <tt>telex</tt>
  # * <tt>national</tt>
  #
  # Calling Display Information Type possible remapped values (<tt>calling_display_type</tt>):
  # * <tt>unspecified</tt> => Type is unspecified.
  # * <tt>calling_party_name</tt> => Type is 0xB1.
  #
  # Those values will be remapped to the protocol specific  Display Information Type value. To provide protocol specific value:
  # * <tt>remapped_fields[:calling_display_type] = 0xB1</tt>
  # or
  # * <tt>remapped_fields[:calling_display_type] = 177</tt>
  #
  # Calling Display Information can be set to any string value (<tt>calling_display</tt>):
  # * <tt>remapped_fields[:calling_display] = "Roger Fluffy"</tt>
  #
  # Calling Presentation possible remapped values (<tt>calling_presentation</tt>):
  # * <tt>unspecified</tt>
  # * <tt>not_available</tt>
  # * <tt>allowed</tt>
  # * <tt>restricted</tt>
  # * <tt>addr_restricted</tt>
  # * <tt>name_restricted</tt>
  #
  # Calling Number Screening possible remapped values (<tt>calling_screening</tt>):
  # * <tt>unspecified</tt>
  # * <tt>no</tt>
  # * <tt>pass</tt>
  # * <tt>fail</tt>
  # * <tt>network_provided</tt>
  #
  def prefix_private_noa( route, caf_call, nap_list, call_params )

    # If the called number have previously been remapped before the call to this function, use it.
    # Otherwise, use the call called number.
    called = call_params[:called]
    calling = call_params[:calling]
    private_address = call_params[:private_address]
    # calling_name = call_params[:private_display]
    # log_trace 2, "Incoming call info =#{call_params.inspect}"
   # puts calling, private_address, calling_name
   # NOA and manipulation for called number
   if @national_regex.match(called)
      call_params[:called_noa] = :national_number
      call_params[:called] = $2 + "F"
   else
      call_params[:called_noa] = :international_number
      call_params[:called] = call_params[:called] + "F"     
   end
   
   # NOA and manipulation for calling number
   # If private_address, then use it as the calling number
   if private_address && @valid_private_address.match(private_address)
      calling = $1
      #log_trace 2, "calling use private_address: calling=#{calling}"
   end
   
    if @national_regex.match(calling)
      call_params[:calling_noa] = :national_number
      call_params[:calling] = $2
    else
      call_params[:calling_noa] = :international_number
    end
  
    #log_trace 2, "Incoming leg called=#{called}, calling=#{calling}, private_address=#{private_address}"
  
    call_params[:called_npi] = :isdn
    call_params
  end

  # Class initialization. Create a class instance variable <tt>regex</tt> hash used by remap_called_noa_npi.
  # * <tt>international_regex</tt> - Math an international called number prefixed with '001'
  #
  def initialize
    @national_regex = Regexp.new(/^(49)([0-9]+)$/)
    #@valid_number_regex = Regexp.new(/^[0-9]+$/)
  #@@request_uri_regex = /sip:(\d*)(;.*)?@.*/
  @valid_private_address = Regexp.new(/sip:(.*)@.*/)
  end

end

@@routing = NoaNpiRemap.new

def init_routes( routes )
  @@routing.init routes
end

def route( call, nap_list )
  @@routing.route call, nap_list
end